@@ -3,7 +3,7 @@ use 5.008001;
 use strict;
 use warnings;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use CPAN::Meta;
 use CPAN::Mirror::Tiny::Archive;
@@ -17,6 +17,8 @@ use File::Spec;
 use File::Temp ();
 use HTTP::Tinyish;
 use Parse::LocalDistribution;
+use Digest::MD5 ();
+use JSON ();
 
 sub new {
     my ($class, %option) = @_;
@@ -77,7 +79,7 @@ sub _system {
 sub inject {
     my ($self, $url, $option) = @_;
     if ($url =~ /(?:^git:|\.git(?:@(.+))?$)/) {
-        $self->inject_git($url, { %{$option || +{}}, $1 ? (ref => $1) :() });
+        $self->inject_git($url, $option);
     } elsif ($url =~ /^https?:/) {
         $self->inject_http($url, $option);
     } else {
@@ -107,7 +109,8 @@ sub inject_http {
     my $file = "$tempdir/$basename";
     my $res = $self->http->mirror($url => $file);
     if ($res->{success}) {
-        return $self->inject_local($file, $option);
+        my $author = ($option ||= {})->{author} || "VENDOR";
+        return $self->_locate_tarball($file, $author);
     } else {
         die "Couldn't get $url: $res->{status} $res->{reason}";
     }
@@ -115,7 +118,15 @@ sub inject_http {
 
 sub inject_git {
     my ($self, $url, $option) = @_;
+
     my $ref = ($option ||= {})->{ref};
+    if ($url =~ /(.*)\@(.*)$/) {
+        # take care of git@github.com:skaji/repo@tag, http://user:pass@example.com/foo@tag
+        my $remove = $2;
+        $ref ||= $remove;
+        $url =~ s/\@$remove$//;
+    }
+
     my $guard = $self->pushd_tempdir;
     my ($ok, $error) = $self->_system("git", "clone", $url, ".");
     die "Couldn't git clone $url: $error" unless $ok;
@@ -127,7 +138,7 @@ sub inject_git {
     die "Couldn't find $metafile in $url" unless -f $metafile;
     my $meta = CPAN::Meta->load_file($metafile);
     chomp(my $rev = `git rev-parse --short HEAD`);
-    my $distvname = sprintf "%s-%s", $meta->name, $rev;
+    my $distvname = sprintf "%s-%s-%s", $meta->name, $meta->version, $rev;
     ($ok, $error) = $self->_system(
         "git archive --format=tar --prefix=$distvname/ HEAD | gzip > $distvname.tar.gz"
     );
@@ -138,11 +149,42 @@ sub inject_git {
     }
 }
 
+my $JSON = JSON->new->canonical(1)->utf8(1);
+
+sub _cached {
+    my ($self, $path, $sub) = @_;
+    my $cache_dir = $self->base("modules", ".cache");
+    File::Path::mkpath($cache_dir) unless -d $cache_dir;
+    my $cache_file = File::Spec->catfile($cache_dir, Digest::MD5::md5_hex($path) . ".json");
+
+    my $mtime = (stat $path)[9];
+    if (-f $cache_file) {
+        my $content = do { open my $fh, "<", $cache_file or die; local $/; <$fh> };
+        my $cache = $JSON->decode($content);
+        if ($cache->{mtime} == $mtime) {
+            return $cache->{payload};
+        } else {
+            unlink $cache_file;
+        }
+    }
+    my $result = $sub->();
+    if ($result) {
+        open my $fh, ">", $cache_file or die;
+        my $content = {mtime => $mtime, path => $path, payload => $result};
+        print {$fh} $JSON->encode($content), "\n";
+        close $fh;
+    }
+    $result;
+}
+
 sub extract_provides {
     my ($self, $path) = @_;
-    unless (File::Spec->file_name_is_absolute($path)) {
-        $path = Cwd::abs_path($path);
-    }
+    $path = Cwd::abs_path($path);
+    $self->_cached($path, sub { $self->_extract_provides($path) });
+}
+
+sub _extract_provides {
+    my ($self, $path) = @_;
     my $gurad = $self->pushd_tempdir;
     my $dir = $self->extract($path) or return;
     my $parser = Parse::LocalDistribution->new({ALLOW_DEV_VERSION => 1});
@@ -233,7 +275,7 @@ CPAN::Mirror::Tiny - create partial CPAN mirror (a.k.a. DarkPAN)
 
 CPAN::Mirror::Tiny helps you create partial CPAN mirror (also known as DarkPAN).
 
-=head2 WHY NEW?
+=head1 WHY NEW?
 
 Yes, we already have great CPAN modules which create CPAN mirror.
 
@@ -248,9 +290,11 @@ Unfortunately existing CPAN mirror modules depend on XS modules.
 
 This is why I made CPAN::Mirror::Tiny.
 
-=head2 METHODS
+=head1 METHODS
 
-=head3 C<< my $cpan = CPAN::Mirror::Tiny->new(%option) >>
+=head2 new
+
+  my $cpan = CPAN::Mirror::Tiny->new(%option)
 
 Constructor. C< %option > may be:
 
@@ -266,7 +310,9 @@ Temp directory. Default C<< File::Temp::tempdir(CLEANUP => 1) >>.
 
 =back
 
-=head3 C<< $cpan->inject($source, \%option) >>
+=head2 inject
+
+  $cpan->inject($source, \%option)
 
 Inject C< $source > to our cpan mirror directory. C< $source > is one of
 
@@ -280,24 +326,56 @@ Inject C< $source > to our cpan mirror directory. C< $source > is one of
 
   $cpan->inject('http://example.com/Module.tar.gz', { author => "DUMMY" });
 
-=item * git url (with optional commitish)
+=item * git url (with optional ref)
 
-  $cpan->inject('git://github.com/skaji/Carl.git@0.114', { author => "SKAJI" });
+  $cpan->inject('git://github.com/skaji/Carl.git', { author => "SKAJI", ref => '0.114' });
 
 =back
 
 As seeing from the above examples, you can specify C<author> in C<\%option>.
 If you omit C<author>, default C<VENDOR> is used.
 
-=head3 C<< my $index_string = $cpan->index >>
+=head2 index
+
+  my $index_string = $cpan->index
 
 Get the index (a.k.a. 02packages.details.txt) of our cpan mirror.
 
-=head3 C<< $cpan->write_index( compress => bool ) >>
+=head2 write_index
+
+  $cpan->write_index( compress => bool )
 
 Write the index to C< $base/modules/02packages.details.txt >
 or C< base/modules/02packages.details.txt.gz >.
 
+=head1 TIPS
+
+=head2 How can I install modules in my DarkPAN with cpanm?
+
+L<cpanm> is an awesome CPAN clients. If you want to install modules
+in your DarkPAN with cpanm, there are 2 ways.
+
+First way:
+
+  cpanm --cascade-search \
+    --mirror-index /path/to/darkpan/modules/02packages.details.txt \
+    --mirror /path/to/darkpan \
+    --mirror http://www.cpan.org \
+    Your::Module
+
+Second way:
+
+  cpanm --mirror-only \
+    --mirror /path/to/darkpan \
+    --mirror http://www.cpan.org \
+    Your::Module
+
+I hope that cpanm delegates the process of not only resolving modules
+but also fetching modules to L<CPAN::Common::Index>-like objects entirely.
+Then we can hack cpanm easily.
+
+I believe that cpanm 2.0 also known as L<Menlo> comes with such features!
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2016 Shoichi Kaji <skaji@cpan.org>
